import click
from art import text2art
import keyboard
import asyncio
import subprocess
import os
# Corrected imports with absolute paths
from JINGKE2.utils.manageenv import Setup, Projectoperation
from JINGKE2.utils.external import external_module
from JINGKE2.utils.git import GitHubLoginApp
from JINGKE2.utils.API import Jingke_api_key, get_api, get_name
from JINGKE2.utils.user import User_creation
from time import gmtime, strftime
time = strftime("%Y-%m-%d %H:%M:%S", gmtime())

@click.command()
@click.option('--init', '-i', is_flag=True, help=': Setup Jing ke for you.')
@click.option('--setup', '-s', is_flag=True, help=': Setup a project in the Environment.')
@click.option('--view', '-v', is_flag=True, help=': View the projects.')
@click.option('--load', '-l', is_flag=True, help=': Load the selected project.')
@click.option('--remove', '-rm', is_flag=True, help=': Remove the Project.')
@click.option('--vscode', '-vs', is_flag=True, help=': To Open the project in VS Code.')
@click.option('--git', '-git', is_flag=True, help=': Push the code into GITHUB.')
def Instruction(setup, view, load, remove, vscode, git, init):
    if setup:
        print("\n------------------------", "Setup Project >>>", "------------------------\n")
        if controller() == 200:
            a = Setup()
        else:
            print("Config your API,\nCheck your network connection.")
    elif view:
        print("\n------------------------", "View Projects >>>", "------------------------\n")
        if controller() == 200:
            Projectoperation.ViewProject()
        else:
            print("Config your API,\nCheck your network connection.")
    elif load:
        print("\n------------------------", "Load Project >>>", "------------------------\n")
        if controller() == 200:
            a = Projectoperation.LoadProject()
        else:
            print("Config your API,\nCheck your network connection.")
        print(a)
    elif remove:
        print("\n------------------------", "Remove a Project >>>", "------------------------\n")
        if controller() == 200:
            a = Projectoperation.RemoveProject()
            print(f"\nProject removed : {a}\n")
        else:
            print("Config your API,\nCheck your network connection.")
    elif vscode:
        print("\n------------------------", "Open Project in VS Code >>>", "------------------------\n")
        Projectoperation.Code()
    elif git:
        print("\n------------------------", "Push to GitHub >>>", "------------------------\n")
        if controller() == 200:
            project_list = Projectoperation.getEnvfolder()
            print(project_list)
            path = external_module.optionSelector(project_list)
            print(f"{path} is selected")
            new_path = os.path.join(os.getcwd(), "Environment", path)
            GitHubLoginApp.create_git(new_path)
        else:
            print("Config your API,\nCheck your network connection.")
    elif init:
        User_creation.initialize()

def main():
    Art = text2art("JINGKE 2.0")
    print(Art)
    Instruction()

def controller():
    api_key = get_api()  # Fetch API key from file or prompt user
    if api_key:
        status = Jingke_api_key(api_key)
        return status

if __name__ == "__main__":
    main()



import sys
import asyncio
from rich.console import Console
from rich.prompt import Prompt

console = Console()

class external_module:
    @staticmethod
    def optionSelector(options: list) -> str | None:
        """
        Display a list of options and allow the user to select one.
        """
        console.print("Select an option:")
        for i, option in enumerate(options):
            console.print(f"{i+1}. {option}")
        console.print(f"{len(options)+1}. Exit")
        choice = Prompt.ask("Choose an option: ", choices=[str(i+1) for i in range(len(options)+1)])
        if choice == str(len(options)+1):
            return None
        return options[int(choice) - 1]

    @staticmethod
    async def loading_animation(message: str, duration: int):
        """
        Display a loading animation asynchronously.

        Args:
            message (str): The message to display before the animation.
            duration (int): The duration of the animation in seconds.
        """
        animation = "|/-\\"
        print(message, end="", flush=True)
        for i in range(duration * 10):
            sys.stdout.write(f"\r{message} {animation[i % len(animation)]}")
            sys.stdout.flush()
            await asyncio.sleep(0.1)
        sys.stdout.write("\r" + " " * (len(message) + 2) + "\r")
    
    @staticmethod
    async def loading():
        await asyncio.gather(
            external_module.loading_animation("Please Wait!", 5),
            asyncio.sleep(5)
        )
    async def fix():
        await asyncio.gather(
            external_module.loading_animation("Fixing issue ⚠️\t", 5),
            asyncio.sleep(5)
        )


import os
import requests
import subprocess
import json
from colorama import Fore, Style, init
import inquirer
import shutil
import tkinter as tk
from tkinter import messagebox
from PIL import Image, ImageTk
import requests
from io import BytesIO

# Initialize colorama
init(autoreset=True)

class GitHubLoginApp:
    @staticmethod
    def check_repo_exists(repo_name, token, username=None):
        """
        Check if a repository already exists for the user.
        """
        # If username not provided, get it from GitHub API
        if not username:
            user_url = "https://api.github.com/user"
            user_headers = {
                "Authorization": f"token {token}",
                "Accept": "application/vnd.github.v3+json"
            }
            user_response = requests.get(user_url, headers=user_headers)
            if user_response.status_code == 200:
                username = user_response.json()["login"]
            else:
                print(f"{Fore.RED}Failed to get user information.")
                return False, None
        
        # Check if the repository exists
        repo_url = f"https://api.github.com/repos/{username}/{repo_name}"
        headers = {
            "Authorization": f"token {token}",
            "Accept": "application/vnd.github.v3+json"
        }
        
        response = requests.get(repo_url, headers=headers)
        
        if response.status_code == 200:
            return True, response.json()["clone_url"]
        else:
            return False, None

    @staticmethod
    def create_github_repo(repo_name, token, description="", private=False):
        """
        Create a new GitHub repository using the GitHub API.
        """
        url = "https://api.github.com/user/repos"
        headers = {
            "Authorization": f"token {token}",
            "Accept": "application/vnd.github.v3+json"
        }
        data = {
            "name": repo_name,
            "description": description,
            "private": private
        }
        
        response = requests.post(url, headers=headers, data=json.dumps(data))
        
        if response.status_code == 201:
            print(f"{Fore.GREEN}Repository '{repo_name}' created successfully!")
            return response.json()["clone_url"]
        else:
            print(f"{Fore.RED}Failed to create repository. Status code: {response.status_code}")
            print(f"{Fore.RED}Response: {response.text}")
            return None

    @staticmethod
    def is_git_repo(folder_path):
        """
        Check if the folder is already a git repository
        """
        original_dir = os.getcwd()
        try:
            os.chdir(folder_path)
            result = subprocess.run(
                ["git", "rev-parse", "--is-inside-work-tree"],
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True
            )
            return result.returncode == 0
        except Exception:
            return False
        finally:
            os.chdir(original_dir)

    @staticmethod
    def remove_git_folder(path):
        """
        Cross-platform way to remove .git folder
        """
        git_folder = os.path.join(path, ".git")
        if os.path.exists(git_folder):
            try:
                shutil.rmtree(git_folder)
                return True
            except Exception as e:
                print(f"{Fore.RED}Error removing .git folder: {e}")
                return False
        return True
    
    @staticmethod
    def handle_venv_gitignore(folder_path):
        """
        Check if the folder contains a venv directory and add it to .gitignore if needed
        """
        # Check if venv folder exists
        venv_folder = os.path.join(folder_path, "venv")
        if os.path.exists(venv_folder) and os.path.isdir(venv_folder):
            gitignore_path = os.path.join(folder_path, ".gitignore")
            
            # Check if .gitignore exists and if venv is already in it
            venv_already_ignored = False
            if os.path.exists(gitignore_path):
                with open(gitignore_path, 'r') as f:
                    content = f.read()
                    # Check for common venv patterns in gitignore
                    venv_patterns = ["/venv/", "venv/", "/venv", "venv"]
                    for pattern in venv_patterns:
                        if pattern in content:
                            venv_already_ignored = True
                            break
            
            # Add venv to .gitignore if needed
            if not venv_already_ignored:
                print(f"{Fore.YELLOW}Found 'venv' folder. Adding to .gitignore...")
                
                # Prepare content to append
                append_content = "\n# Virtual Environment\nvenv/\n"
                
                # Ensure we don't add duplicate newlines
                if os.path.exists(gitignore_path):
                    with open(gitignore_path, 'r') as f:
                        existing_content = f.read()
                    if not existing_content.endswith('\n'):
                        append_content = '\n' + append_content
                
                # Append to .gitignore
                with open(gitignore_path, 'a+') as f:
                    f.write(append_content)
                print(f"{Fore.GREEN}Successfully added 'venv/' to .gitignore")
            else:
                print(f"{Fore.CYAN}'venv' already exists in .gitignore, no changes needed")

    @staticmethod
    def push_folder_to_github(folder_path, repo_url, token, force_reinit=False, force_add=True):
        """
        Initialize a Git repository in the folder (if needed) and push it to GitHub.
        
        Args:
            folder_path: Path to the folder to push
            repo_url: URL of the GitHub repository
            token: GitHub personal access token
            force_reinit: Force reinitialization even if it's a git repo
            force_add: Force add files even if they are in .gitignore
        """
        # Add token to the URL for authentication
        repo_url_with_token = repo_url.replace("https://", f"https://{token}@")
        
        # Save current directory
        original_dir = os.getcwd()
        
        try:
            # Change to the specified folder
            os.chdir(folder_path)
            
            # Check and handle venv folder in gitignore
            GitHubLoginApp.handle_venv_gitignore(folder_path)
            
            # Check if it's already a git repository
            is_repo = GitHubLoginApp.is_git_repo(".")
            
            if not is_repo or force_reinit:
                if is_repo and force_reinit:
                    print(f"{Fore.YELLOW}Reinitializing existing git repository...")
                    # Use cross-platform method to remove .git folder
                    GitHubLoginApp.remove_git_folder(".")
                
                # Initialize Git repository
                print(f"{Fore.CYAN}Initializing git repository...")
                subprocess.run(["git", "init"], check=True)
                
                # Configure remote
                print(f"{Fore.CYAN}Adding remote origin...")
                try:
                    subprocess.run(["git", "remote", "add", "origin", repo_url_with_token], check=True)
                except subprocess.CalledProcessError:
                    # If remote already exists, set its URL
                    subprocess.run(["git", "remote", "set-url", "origin", repo_url_with_token], check=True)
            else:
                # Repository already exists, just update the remote URL
                print(f"{Fore.CYAN}Using existing git repository...")
                try:
                    subprocess.run(["git", "remote", "add", "origin", repo_url_with_token], check=True)
                except subprocess.CalledProcessError:
                    # If remote already exists, set its URL
                    subprocess.run(["git", "remote", "set-url", "origin", repo_url_with_token], check=True)
            
            # Set Git config for first-time users if needed
            try:
                # Check if user.email is set
                email_check = subprocess.run(
                    ["git", "config", "user.email"],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    text=True
                )
                
                if not email_check.stdout.strip():
                    email = input(f"{Fore.YELLOW}Enter your email for Git commits: ")
                    subprocess.run(["git", "config", "user.email", email], check=True)
                    
                # Check if user.name is set
                name_check = subprocess.run(
                    ["git", "config", "user.name"],
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    text=True
                )
                
                if not name_check.stdout.strip():
                    name = input(f"{Fore.YELLOW}Enter your name for Git commits: ")
                    subprocess.run(["git", "config", "user.name", name], check=True)
            except Exception as e:
                print(f"{Fore.YELLOW}Warning: Unable to set Git user config: {e}")
            
            # Add all files - use -f to force adding files that might be in .gitignore
            print(f"{Fore.CYAN}Adding files...")
            add_command = ["git", "add", "."]
            if force_add:
                add_command = ["git", "add", "-f", "."]
                print(f"{Fore.YELLOW}Forcing add of all files (including ignored files)...")
            
            try:
                subprocess.run(add_command, check=True)
            except subprocess.CalledProcessError as e:
                print(f"{Fore.YELLOW}Warning: Some files may be ignored: {e}")
                
                # Ask user if they want to force add
                if not force_add:
                    questions = [
                        inquirer.List('force_add',
                                     message="Some files may be ignored. Force add all files?",
                                     choices=['Yes', 'No'],
                                     default='Yes'),
                    ]
                    answers = inquirer.prompt(questions)
                    if answers['force_add'] == 'Yes':
                        print(f"{Fore.CYAN}Forcing add of all files...")
                        subprocess.run(["git", "add", "-f", "."], check=True)
                    else:
                        print(f"{Fore.YELLOW}Using standard git add, some files may not be included...")
                        # Use a different approach - add only tracked files or those not in .gitignore
                        subprocess.run(["git", "add", "-u"], check=True)  # Update tracked files
            
            # Check if there are changes to commit
            status = subprocess.run(
                ["git", "status", "--porcelain"],
                stdout=subprocess.PIPE,
                text=True
            )
            
            if status.stdout.strip():
                print(f"{Fore.CYAN}Committing changes...")
                try:
                    subprocess.run(["git", "commit", "-m", "Update project files"], check=True)
                except subprocess.CalledProcessError as e:
                    print(f"{Fore.RED}Commit failed: {e}")
                    
                    # If commit fails, try to set up git user config and retry
                    try:
                        email = input(f"{Fore.YELLOW}Commit failed. Enter your email for Git commits: ")
                        name = input(f"{Fore.YELLOW}Enter your name for Git commits: ")
                        subprocess.run(["git", "config", "user.email", email], check=True)
                        subprocess.run(["git", "config", "user.name", name], check=True)
                        subprocess.run(["git", "commit", "-m", "Update project files"], check=True)
                    except subprocess.CalledProcessError:
                        print(f"{Fore.RED}Failed to commit changes even after setting user config")
                        raise
            else:
                print(f"{Fore.YELLOW}No changes to commit")
            
            # Push to GitHub
            print(f"{Fore.CYAN}Pushing to GitHub...")
            try:
                # Try to push to the current branch
                current_branch = subprocess.run(
                    ["git", "branch", "--show-current"],
                    stdout=subprocess.PIPE,
                    text=True,
                    check=True
                ).stdout.strip()
                
                if not current_branch:
                    current_branch = "main"  # Default to main if no branch is found
                    
                subprocess.run(["git", "push", "-u", "origin", current_branch], check=True)
                
            except subprocess.CalledProcessError:
                # If the push fails, try pushing to main/master
                try:
                    print(f"{Fore.YELLOW}Push failed, trying main branch...")
                    subprocess.run(["git", "push", "-u", "origin", "main"], check=True)
                except subprocess.CalledProcessError:
                    print(f"{Fore.YELLOW}Push to main failed, trying master branch...")
                    try:
                        subprocess.run(["git", "push", "-u", "origin", "master"], check=True)
                    except subprocess.CalledProcessError:
                        # As a last resort, try force push
                        print(f"{Fore.YELLOW}Standard push failed, attempting force push (use with caution)...")
                        try:
                            subprocess.run(["git", "push", "-f", "-u", "origin", current_branch or "main"], check=True)
                        except subprocess.CalledProcessError as e:
                            print(f"{Fore.RED}All push attempts failed: {e}")
                            raise
            
            print(f"{Fore.GREEN}Successfully pushed folder '{folder_path}' to GitHub repository!")
            
        except subprocess.CalledProcessError as e:
            print(f"{Fore.RED}Git operation failed: {e}")
        finally:
            # Change back to original directory
            os.chdir(original_dir)

    @staticmethod
    def create_git(folder_path):
        """
        Main function to handle GitHub repository creation and pushing.
        This is what will be called from the CLI.
        """
        print(f"{Fore.CYAN}GitHub Repository Setup")
        print(f"{Fore.CYAN}======================")
        
        dire = os.path.join(os.getcwd(),"settings")
        file = os.path.join(dire, "project_info.json")
        try:
            with open(file,'r') as f:
                data = json.load(f)
                token = data.get("TOKEN")
        except:
            token = ""        

        if not token:
            # Get GitHub token
            root = tk.Tk()
            app = GitHubUI(root)
            root.mainloop()
            #token = input(f"{Fore.YELLOW}Enter your GitHub token: ")
        
        # Get repository name (default to folder name)
        folder_name = os.path.basename(folder_path)
        repo_name = input(f"{Fore.YELLOW}Enter repository name [{folder_name}]: ") or folder_name
        
        # Get repository description
        repo_description = input(f"{Fore.YELLOW}Enter repository description (optional): ")
        
        # Ask if private
        questions = [
            inquirer.List('private',
                          message="Should the repository be private?",
                          choices=['No', 'Yes'],
                          default='No'),
        ]
        answers = inquirer.prompt(questions)
        is_private = answers['private'] == 'Yes'
        
        # Check if repository already exists
        print(f"{Fore.CYAN}Checking if repository already exists...")
        exists, repo_url = GitHubLoginApp.check_repo_exists(repo_name, token)
        
        if exists:
            print(f"{Fore.GREEN}Repository '{repo_name}' already exists, using it.")
        else:
            # Create GitHub repository
            print(f"{Fore.YELLOW}Repository '{repo_name}' doesn't exist, creating it.")
            repo_url = GitHubLoginApp.create_github_repo(repo_name, token, repo_description, is_private)
        
        if repo_url:
            # Ask if want to force reinitialize
            questions = [
                inquirer.List('reinit',
                            message="Force reinitialize git repository (if it exists)?",
                            choices=['No', 'Yes'],
                            default='No'),
            ]
            answers = inquirer.prompt(questions)
            force_reinit = answers['reinit'] == 'Yes'
            
            # Ask if want to force add files
            questions = [
                inquirer.List('force_add',
                            message="Force add all files (including ignored files)?",
                            choices=['Yes', 'No'],
                            default='Yes'),
            ]
            answers = inquirer.prompt(questions)
            force_add = answers['force_add'] == 'Yes'
            
            # Push folder to GitHub
            GitHubLoginApp.push_folder_to_github(folder_path, repo_url, token, force_reinit, force_add)
        else:
            print(f"{Fore.RED}Failed to get or create repository. Cannot push folder.")

class GitHubUI:
    def __init__(self, root):
        self.root = root
        self.root.title("GitHub Login")
        self.root.geometry("400x350")
        self.root.resizable(False, False)
        self.root.configure(bg="white")

        # Load and display the GitHub logo
        try:
            logo_url = "https://github.githubassets.com/images/modules/logos_page/GitHub-Mark.png"
            response = requests.get(logo_url)
            logo_image = Image.open(BytesIO(response.content))
            logo_image = logo_image.resize((100, 100), Image.ANTIALIAS)
            logo_photo = ImageTk.PhotoImage(logo_image)
            self.logo_label = tk.Label(root, image=logo_photo)
            self.logo_label.image = logo_photo  # Keep a reference to prevent garbage collection
            self.logo_label.pack(pady=10)
        except Exception as e:
            messagebox.showerror("Error", f"Failed to load logo: {e}")

        # Create a frame for the login form
        self.frame = tk.Frame(root, padx=20, pady=20)
        self.frame.pack(expand=True)
        self.frame.configure(bg="white")

        # Title Label
        self.title_label = tk.Label(self.frame, text="GitHub Login", font=("Arial", 16, "bold"))
        self.title_label.configure(bg="white")
        self.title_label.grid(row=0, column=0, columnspan=2, pady=10)

        # Username Label and Entry
        self.username_label = tk.Label(self.frame, text="Username:", font=("Arial", 12))
        self.username_label.configure(bg="white")
        self.username_label.grid(row=1, column=0, sticky="w", pady=5)

        self.username_entry = tk.Entry(self.frame, width=30, font=("Arial", 12))
        self.username_entry.grid(row=1, column=1, pady=5)

        # Password Label and Entry
        self.password_label = tk.Label(self.frame, text="Token:", font=("Arial", 12))
        self.password_label.configure(bg="white")
        self.password_label.grid(row=2, column=0, sticky="w", pady=5)

        self.password_entry = tk.Entry(self.frame, width=30, show="*", font=("Arial", 12))
        self.password_entry.grid(row=2, column=1, pady=5)

        # Login Button
        self.login_button = tk.Button(self.frame, text="Login", font=("Arial", 12), bg="#4CAF50", fg="white", command=self.validate_login)
        self.login_button.grid(row=3, column=0, columnspan=2, pady=20)

    def validate_login(self):
        try:
            username = self.username_entry.get()
            password = self.password_entry.get()

            path = os.path.join(os.getcwd(),"settings")
            if not os.path.exists(path):
                os.makedirs(path)
            data = {
                "USER_NAME" : username,
                "TOKEN" : password
            }
            json_file_path = os.path.join(path, "project_info.json")
            with open(json_file_path, "w") as json_file:
                json.dump(data, json_file, indent=4)
            messagebox.showinfo("Success", f"Information Added. You can close the login menu")
            self.username_entry.delete(0, tk.END)
            self.password_entry.delete(0, tk.END)

        except Exception as e:
            messagebox.showerror("Error", f"An error occurred: {e}")



if __name__ == '__main__':
    root = tk.Tk()
    app = GitHubUI(root)
    root.mainloop()


import os
class Writer:
    def pycreate(path):
        file = 'main.py'
        dir_list = os.listdir(path)
        print(dir_list) 
        with open(os.path.join(path, file), 'w') as fp:
            pass 
        dir_list = os.listdir(path)
    
    def writeCode(code, path):
        file = 'main.py'
        with open(os.path.join(path, file), 'w') as fp:
            fp.write(code)
     
    


import requests
import json
import os
from art import *
from JINGKE2.utils.user import User_creation

def verify_api_key(api_key):
    """
    Verify if the API key is valid by calling the dynamic Next.js API route.

    Args:
        api_key (str): The API key to verify.

    Returns:
        dict: The response from the API.
    """
    url = f"https://jingke-server.vercel.app/api/verify/{api_key}"  # Dynamic API route

    try:
        response = requests.get(url)  # No explicit method like POST
        response.raise_for_status()  # Raise error if request fails

        return response.json() if response.text.strip() else {"success": False, "message": "Empty response"}

    except requests.exceptions.JSONDecodeError:
        print("Error: Response is not valid JSON.")
        print("Response text:", response.text)
        return {"success": False, "message": "Invalid JSON response"}

    except requests.exceptions.RequestException as e:
        print(f"Error: Unable to connect to API. {str(e)}")
        return {"success": False, "message": str(e)}

def Jingke_api_key(api_key):
    result = verify_api_key(api_key)
    if result.get("success"):
        return 200
    else:
        print(f"Error: {result.get('message')}")
        return 404

def get_api():
    settings_dir = os.path.join(os.getcwd(), "settings")
    file_path = os.path.join(settings_dir, "profile.json")
    if not os.path.exists(file_path):
        User_creation.initialize()
    try:
        with open(file_path, 'r') as f:
            data = json.load(f)
            return data.get("api")
    except (FileNotFoundError, json.JSONDecodeError):
        print("\nError: Unable to load API key from profile.json.\n")
        return None

def get_name():
    settings_dir = os.path.join(os.getcwd(), "settings")
    file_path = os.path.join(settings_dir, "profile.json")

    try:
        with open(file_path, 'r') as f:
            data = json.load(f)
            return data.get("name")
    except (FileNotFoundError, json.JSONDecodeError):
        return None



from langchain_ollama import ChatOllama
from langchain.prompts import ChatPromptTemplate
import os
import subprocess
import json
from JINGKE2.utils.codeeditor import Writer

class ErrorId:
    @staticmethod
    def get_last_code(file_path, key) -> str:
        with open(file_path, 'r') as file:
            data = json.load(file)
            last_code = data[key]
            return last_code
        
    @staticmethod
    def gettype(path, model):
        json_name = "query.json"
        temp_loc = os.path.join(path, json_name)
        chat = ChatOllama(model=model)
        error_file = os.path.join(path, "output.txt")
        
        with open(error_file, 'r') as f:
            error_output = f.read().lower()
        
        # Direct error pattern matching
        if "modulenotfounderror" in error_output:
            return "e1"
        
        # LLM classification for other errors
        prompt_template = ChatPromptTemplate.from_messages([
            ("system", """Analyze this error strictly. Return ONLY the error code:
            - 'e1' ONLY for ModuleNotFoundError
            - 'e2' for ImportError
            - 'e3' for PermissionError
            - 'e0' for any other error
            No explanations, just the code."""),
            ("human", "Error log: {error_log}")
        ])
        
        chain = prompt_template | chat
        return chain.invoke({"error_log": error_output}).content

    
    @staticmethod
    def ModuleFix(path, model):
        chat = ChatOllama(model=model)
        error = os.path.join(path, "output.txt")

        with open(error, 'r') as f:
            output = f.read()

        template = f""" 
            Provide only the Windows simple command to fix this issue: {output}. Do not include any explanation or commands for Linux or Mac. Do not use any markdown code blocks. Only output the command to install dependencies, and avoid any other extraneous information.
            do not use "``````" while generating the code.
        """

        system = template
        human = "{text}"
        prompt = ChatPromptTemplate.from_messages([("system", system), ("human", human)])
        chain = prompt | chat

        fix = chain.invoke("").content.strip()

        venv_dir = os.path.join(path, "venv")
        python_exe = os.path.join(venv_dir, "Scripts", "python.exe")

        subprocess.run([python_exe, "-m", "ensurepip"], check=True)
        subprocess.run([python_exe, "-m", "pip", "install", "--upgrade", "pip"], check=True)

        command = fix.split()
        subprocess.run([python_exe, "-m"] + command, check=True)

        print("\nFixing More Issues ...\n\n")

    @staticmethod
    def CodeFix(path,model)-> None:
        chat = ChatOllama(model=model)
        error = os.path.join(path, "output.txt")
        json_name = "query.json"
        temp_loc = os.path.join(path, json_name)
        code = ErrorId.get_last_code(temp_loc, "response")
        with open(error, 'r') as f:
            output = f.read()

        template = f"""
        I have a code : {code}. I got this output: {output}.  
        rewrite the code to fix this error. 
        Only provide the code in python, no explanations, no headers, and do not use "``````" while generating the code.
            avoid explanation and don't use "``````" even if the context is large.
            avoid this "Here is the code with all comments removed:
                ```python"
                avoid this at the end of the code "```"
        """

        system = template
        human = "{text}"
        prompt = ChatPromptTemplate.from_messages([("system", system), ("human", human)])
        chain = prompt | chat

        fix = chain.invoke("").content.strip()
        Writer.writeCode(code,path)
        print("\nFixing More Issues ...\n\n")




from setuptools import setup, find_packages

# Read dependencies from requirements.txt
def read_requirements():
    try:
        with open("requirements.txt", encoding="utf-8") as f:
            return [line.strip() for line in f if line.strip() and not line.startswith("#")]
    except FileNotFoundError:
        return []

setup(
    name="jingke",
    version="1.1.5",
    packages=find_packages(),
    install_requires=read_requirements(),  # Load dependencies dynamically
    entry_points={
        "console_scripts": [
            "jingke = JINGKE2.main:main",  # Calls the `main` function in main.py
        ],
    },
    author="R Kiran Kumar Reddy",
    author_email="ki2003167@gmail.com",
    description="A Python Developer, Used to generate, debug and upload the code in GITHUB.",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
    ],
    python_requires=">=3.10",
)



import asyncio
from langchain.prompts import ChatPromptTemplate
from langchain_ollama import ChatOllama
from JINGKE2.utils.codeeditor import Writer
from JINGKE2.utils.external import external_module
from JINGKE2.utils.Code.coderunner import Runner
from JINGKE2.utils.test.fixissues import ErrorId
import subprocess
import os
import json
import re
import sys

class LLM:
    def __init__(self):
        pass
    
    def get_python_code(response):
        code_blocks = re.findall(r"```python\n(.*?)```", response, re.DOTALL)
        return "\n\n".join(code_blocks) if code_blocks else response  # Fallback if no markdown format


    @staticmethod
    def get_last_code(file_path, key) -> str:
        with open(file_path, 'r') as file:
            data = json.load(file)
            last_code = data[key]
            return last_code
    
    @staticmethod
    async def error_fixer_ModuleNotFound(path,model) -> None:
        """output_path = os.path.join(path,"output.txt") 
        with open(output_path, 'r') as f:
            output = f.read()"""
        r = ErrorId.gettype(path,model)
        print(r)
        if r == 'e1' or r == 'E1' :
            await external_module.fix()
            ErrorId.ModuleFix(path,model)
            Runner.coderunner(path)
            await LLM.error_fixer_ModuleNotFound(path,model)
        elif r == 'e2' or r == 'E2':
            await external_module.fix()
            ErrorId.CodeFix(path,model)
            await LLM.error_fixer_ModuleNotFound(path,model)
        else:
            print("\n------------------------", f"<< Error Fixed ✅ >>", "------------------------\n")

    @staticmethod
    async def JinOllama(model: str, path: str) -> str | None:
        """
        Asynchronous method to process a query using ChatOllama and write the response to a file.
        """
        chat = ChatOllama(model=model, temperature=0.5)
        prompt_template = f"""  
        Only provide the code in python, no explanations, no headers, and do not use "``````" while generating the code.
        your project location is {path}. No need to mention the path inside the code, if you want to then comment it out.
        """
        print("\n\n------------------------", f"<< Working on {path} >>", "------------------------\n")
        await external_module.loading()  # Changed from asyncio.run()
        json_name = "query.json"
        temp_loc = os.path.join(path, json_name)
        main_pot = os.path.join(path,"main.py")
        if not os.path.exists(main_pot):
            # Prepare the prompt
            #save the json, if is not created
            if not os.path.exists(temp_loc):
                query = input("Prompt : ")
                query= query+f"my project location is {path}. If you save anything save here. If loaction required, then :Use full location, if you want to store anything. If not required : No need to use"
                system = prompt_template
                human = "{text}"
                prompt = ChatPromptTemplate.from_messages([("system", system), ("human", human)])
                chain = prompt | chat
                # Invoke the chain asynchronously
                response = await chain.ainvoke({"text": query})  # Changed to ainvoke
                res = response.content
                # Install all required dependencies:
                a = LLM.get_python_code(res)
                # Write the response to the specified path
                Writer.writeCode(a, path)
                print(f"Code has been written to {path}")
                data = {
                    "prompt": query,
                    "response": a
                }
                """python_file = os.path.join(path,"main.py")
                command = f"start cmd /k {path}/venv/Scripts/python.exe {python_file}"
                result = subprocess.Popen(command, shell=True)
                py_location = os.path.join(path,"venv/Scripts/python.exe")
                script_path = os.path.join(path, "main.py")
                file_path = os.path.join(path, "error.txt")
                with open(file_path, 'w') as f:
                    subprocess.run([py_location, script_path], stderr=f)
                with open(file_path, 'r') as f:
                    output = f.read()"""
                with open(temp_loc, 'w') as f:
                    json.dump(data, f, indent=4)
                # run the code
                Runner.coderunner(path)
                await LLM.error_fixer_ModuleNotFound(path,model)
        # If the query is empty, return None 
        # If the json file contains the response, then it will run the 
        if os.path.exists(temp_loc):
            query = input("Filtered Prompt : ")
            q1= query+f"my project location is {path}. If you save anything save here. If loaction required, then :Use full location, if you want to store anything. If not required : No need to use"
            last_prompt = LLM.get_last_code(temp_loc, "prompt")
            last_code = LLM.get_last_code(temp_loc, "response")
            prompt_template = f"""  
            Only provide the code in python, no explanations, no headers, and do not use "``````" while generating the code.
            avoid explanation and don't use "``````" even if the context is large.
            avoid this "Here is the code with all comments removed:
                ```python"
                avoid this at the end of the code "```"
            """
            system = prompt_template
            human = "{text}"
            prompt = ChatPromptTemplate.from_messages([("system", system), ("human", human)])
            chain = prompt | chat
            #print(last_code)
            query = f"The last query is {last_prompt},the last code is {last_code}. Now you have to do : {query}"
            response = await chain.ainvoke({"text": query})  # Changed to ainvoke
            a = response.content
            # Write the response to the specified path
            Writer.writeCode(a, path)
            data = {
                "prompt": q1,
                "response": a
            }
            
            with open(temp_loc, 'w') as f:
                json.dump(data, f, indent=4)
                print(f"Code has been written to {path}")
            """python_file = os.path.join(path,"main.py")
            command = f"start cmd /k {path}/venv/Scripts/python.exe {python_file}"
            subprocess.Popen(command, shell=True)
            py_location = os.path.join(path,"venv/Scripts/python.exe")
            script_path = os.path.join(path, "main.py")
            file_path = os.path.join(path, "error.txt")
            with open(file_path, 'w') as f:
            subprocess.run([py_location, script_path], stderr=f)
            with open(file_path, 'r') as f:
                output = f.read()"""
            Runner.coderunner(path)
            #get error type
            #check the error
            await LLM.error_fixer_ModuleNotFound(path,model)
            await LLM.JinOllama(model, path)

